;;; ralee-edit

; Copyright (c) 2004 Sam Griffiths-Jones
;
; This is part of RALEE -- see
; http://www.sanger.ac.uk/Users/sgj/ralee/ and the 00README file
; that should accompany this file.
;
; RALEE is free software; you can redistribute it and/or modify it
; under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; RALEE is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
; General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with RALEE; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA



(defun insert-gap-column ()
  "Insert a column of gap residues"
  (interactive)
  (save-excursion
    (if (ralee-is-alignment-column)
	(let ((column (current-column)))
	  (goto-char (point-min))
	  (while (< (point) (point-max))
	    (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
		(progn
		  (move-to-column column)
		  (insert ralee-gap-symbol)))
	    (forward-line)
	    )
	  )
      (message "Not within the alignment")
      )
    )
  )


(defun insert-gap ()
  "Insert a gap residues"
  (interactive)
  (if (ralee-is-alignment-column)
      (if (not overwrite-mode)
	  (insert ralee-gap-symbol)
	(progn
	  (if (or (not (ralee-is-alignment-line))
		  (char-equal (char-after) ?\.)
		  (char-equal (char-after) ?\-))
	      (progn
		(insert ralee-gap-symbol)
		(delete-char 1)
		)
	    (message "can't overwrite a non-gap character")
	    )
	  )
	)
    (message "Not within the alignment")
    )
  )


(defun delete-gap ()
  "delete a gap residue"
  (interactive)
  (if (ralee-is-alignment-column)
      (if (or (char-equal (char-after) ?\.)
	      (char-equal (char-after) ?\-))
	  (delete-char 1)
	(message "can't delete a non-gap character")
	)
    (message "not within the alignment!")
    )
  )

(defun delete-back-gap ()
  "backspace action but only for gap chars"
  (interactive)
  (if (ralee-is-alignment-column)
      (progn
	(forward-char -1)
	(if (or (char-equal (char-after) ?\.)
		(char-equal (char-after) ?\-))
	    (delete-char 1)
	  (progn
	    (forward-char 1)
	    (message "can't delete a non-gap character")
	    )
	  )
	)
    (message "not within the alignment!")
    )
  )

(defun is-all-gap-column ()
  "test whether the current column is all gaps"
  (save-excursion
    (let (column notagap)
      (setq column (current-column))
      (goto-char (point-min))
      (while (< (point) (point-max))
	(if (or (ralee-is-alignment-line) (ralee-is-markup-line))
	    (progn
	      (move-to-column column)
	      (setq char (char-after))
	      (if (char-equal char (string-to-char ralee-gap-symbol))
		  ()
		(progn
		  (setq notagap t)
		  (goto-char (point-max)) ; break the loop
		  )
		)
	      )
	  )
	(forward-line)
	)
      (if notagap
	  nil
	t
	)
      )
    )
  )



(defun delete-gap-column ()
  "Delete a column but only if it is all gaps"
  (interactive)
  (save-excursion
    (if (ralee-is-alignment-column)
	(let (column)
	  (setq column (current-column))

	  (if (is-all-gap-column)
	      (progn
		(goto-char (point-min))
		(while (< (point) (point-max))
		  (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
		      (progn
			(move-to-column column)
			(delete-char 1)
			)
		    )
		  (forward-line)
		  )
		)
	    (message "Column %s contains non-gap characters -- cannot delete" column)
	    )
	  )
      (message "not within the alignment!")
      )
    )
  )


(defun delete-all-gap-columns ()
  "delete all gapped columns"
  (interactive)
  (save-excursion
    (goto-char (point-min))
    ; find me an alignment line
    (while (not (ralee-is-alignment-line))
     (forward-line)
     )
    (end-of-line)
    (search-backward " ")
    (let ((i 0))
      (while (< (point) (line-end-position))
	(if (is-all-gap-column)
	    (progn
	      (delete-gap-column)
	      (backward-char)
	      (setq i (1+ i))
	      )
	  )
	(forward-char)
	)
      (message "deleted %s all-gapped columns" i)
      )
    )
  )



(defun unblock-alignment ()
  "unblock a blocked alignment"
  (interactive)
  (goto-char (point-min))
  (let ((seqs ())
	seqid
	(ids ())
	seqstr
	(maxidlength 0)
	(split ())
	name)

    (while (< (point) (point-max))
      (beginning-of-line)
      (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
	  (progn
	    (setq seqid (ralee-get-seq-id))
	    (setq seqstr (ralee-get-seq-string))
	    (if (assoc seqid seqs)
		(setq seqstr (concat (cdr (assoc seqid seqs)) seqstr))
	      (progn
		(push seqid ids)
		(setq split (split-string seqid ""))
		(if (> (length split) maxidlength)
		    (setq maxidlength (length split))
		  )
		)
	      )
	    (setq seqs (cons (cons seqid seqstr) seqs))
	    )
	)
      (forward-line)
      )

    ; backup the current buffer and overwrite with unblocked alignment
    (setq name (buffer-name))
    (write-file (concat (buffer-name) ".bak"))
    (get-buffer-create name)
    (switch-to-buffer name)

    (setq ids (reverse ids))

    (insert "# STOCKHOLM 1.0\n\n")
    (while ids
      (setq seqid (car ids))
      (setq ids (cdr ids))

      (setq split (split-string seqid ""))
      (insert seqid)

      (let ((i 0))
	(while (<= i (- (1+ maxidlength) (length split)))
	  (insert " ")
	  (setq i (1+ i))
	  )
	)
      (insert (cdr (assoc seqid seqs)))
      (insert "\n")
      )
    (insert "\/\/\n")

    ;; do this before the insert above and it all goes badly wrong
    (ralee-mode) ; new buffer doesn't inherit mode!
    )
  )


(defun trim-seq (&optional right)
  "trim the current sequence left of cursor position, or right if optional"
  (save-excursion
    (let ((start (point)))
      (if right
	  (end-of-line)
	(search-backward " ")
	(forward-char)
	)
      (kill-region start (point))
      )
    (let (nse
	  offset
	  st
	  en)

      (setq nse (ralee-get-name-start-end))

      (if (and (ralee-is-alignment-line) (nth 1 nse))  ; then seqid includes start end
	  (progn
	    (setq offset (length
			  (split-string 
			   (ralee-ungap-string (car kill-ring)) "")))

            ; GNU emacs: (split-string "AACC" "") => ("A" "A" "C" "C")
            ; xemacs:    (split-string "AACC" "") => ("" "A" "A" "C" "C" "")
	    (if (featurep 'xemacs)
		(setq offset (- offset 2))
	      )

	    (setq st (nth 1 nse))
	    (setq en (nth 2 nse))
	    (if right
		(if (< en st)
		    (setq en (+ en offset))
		  (setq en (- en offset))
		  )
	      (if (< en st)
		  (setq st (- st offset))
		(setq st (+ st offset))
		)
	      )
	    (let ((newnse (concat (nth 0 nse) "/" 
				  (number-to-string st) "-" 
				  (number-to-string en)))
		  (oldnse (concat (nth 0 nse) "/" 
				  (number-to-string (nth 1 nse)) "-" 
				  (number-to-string (nth 2 nse))))
		  oldlen
		  newlen)

	      (setq oldlen (length (split-string oldnse "")))
	      (setq newlen (length (split-string newnse "")))

	      (ralee-find-first-column)
	      (let ((first (point)))
		(beginning-of-line)
		(kill-region (point) first)
		(insert newnse)
		(while (< (point) first)
		  (insert " ")
		  )
		)
	      (list oldnse newnse)  ; return this so we can fix #=GR lines
	      )
	    )
	)
      )
    )
  )



(defun trim-alignment (&optional right)
  "trim the alignment"
  (save-excursion
    (if (ralee-is-alignment-column)
	(let ((column (current-column)))

	  (ralee-find-first-column)
	  (let ((pad 0)
		(maxid (ralee-maxidlength)))
	    (if (< (current-column) (+ maxid 8))
		(setq pad (+ (- (current-column) maxid) 8))
	      )

	    (goto-char (point-min))
	    (while (< (point) (point-max))
	      (if (or (ralee-is-alignment-line) (ralee-is-markup-line))
		  (progn
		    (ralee-find-first-column)
		    (let ((i 0))
		      (while (< i pad)
			(insert " ")
			(setq i (1+ i))
			)
		      )

		    (ralee-find-first-column)
		    (let ((firstcol (current-column)))
		      (move-to-column column)
		      (let ((oldnew (trim-seq right)))  ;; do the trim
			(if oldnew   ; then we need to check for another instance of the
			             ; nse string to replace, maybe a #=GR line
			    (save-excursion
			      (goto-char (point-min))
			      (while (search-forward (nth 0 oldnew) nil t)
				(end-of-line)
				(let ((eol (point)))
				  (beginning-of-line)

				  ;; perform-replace should set limits but I can't get it to work
				  (save-restriction
				    (narrow-to-region (point) eol)
				    (perform-replace (nth 0 oldnew) (nth 1 oldnew) nil nil nil)
				    )
				  )

				;; now make sure we left justify properly
				(move-to-column firstcol)
				(let ((col (current-column)))
				  (ralee-find-first-column)
				  (while (> (current-column) col)
				    (forward-char -1)
				    (delete-char 1)
				    )
				  (while (< (current-column) col)
				    (insert " ")
				    )
				  )
				
				)
			      )
			    )
			)
		      )
		    )
		)
	      (forward-line)
	      )
	    )
	  )
      (message "not within the alignment!")
      )
    )
  )


(defun trim-left ()
  "trim the alignment left of cursor position"
  (interactive)
  (trim-alignment)
  )


(defun trim-right ()
  "trim the alignment right of cursor position"
  (interactive)
  (trim-alignment t)
  )


(defun is-ragged-alignment ()
  "check whether the current alignment is flush
return nil if flush, column number if not"
  (save-excursion
    (ralee-find-first-line)
    (end-of-line)
    (let ((column (current-column))
	  return)
      (while (< (point) (point-max))
	(if (or (ralee-is-alignment-line)
		(ralee-is-markup-line))
	    (progn
	      (end-of-line)
	      (if (< column (current-column))
		  (if (or (equal return nil)
			  (> (current-column) return))
		      (setq return (current-column))
		    )
		)
	      (if (> column (current-column))
		  (if (or (equal return nil)
			  (> column return))
		      (setq return column)
		    )
		)
	      )
	  )
	(forward-line)
	)
      return
      )
    )
  )


(defun make-flush-alignment ()
  "make the alignment flush at the right extent"
  (interactive)
  (save-excursion
    (let ((column (is-ragged-alignment)))
      (if column
	  (progn
	    (ralee-find-first-line)
	    (while (< (point) (point-max))
	      (if (or (ralee-is-alignment-line)
		      (ralee-is-markup-line))
		  (progn
		    (end-of-line)
		    (while (< (current-column) column)
		      (insert ralee-gap-symbol)
		      )
		    )
		)
	      (forward-line)
	      )
	    )
	)
      )
    )
  )


(defun left-justify-alignment ()
  "Make sure left end of sequences are justified properly.
This method is used to clean up after things like the trim methods"
  (interactive)
  (save-excursion
    (let ((maxid (ralee-maxidlength)))
      (goto-char (point-min))
      (ralee-find-first-line)
      (while (< (point) (point-max))
	(if (or (ralee-is-alignment-line) (ralee-is-markup-line))
	    (progn
	      (move-to-column (+ maxid 6))
	      (let ((col (current-column)))
		(ralee-find-first-column)
		(while (> (current-column) col)
		  (forward-char -1)
		  (delete-char 1)
		  )
		(while (< (current-column) col)
		  (insert " ")
		  )
		)
	      )
	  )
	(forward-line)
	)
      )
    )
  )

(defun uppercase-alignment ()
  "make all sequences uppercase"
  (interactive)
  (save-excursion
    (ralee-find-first-line)
    (while (< (point) (point-max))
      (if (ralee-is-alignment-line)
	  (progn 
	    (ralee-find-first-column)
	    (let ((first (point)))
	      (end-of-line)
	      (upcase-region first (point))
	      )
	    )
	)
      (forward-line)
      )
    )
  )

(defun lowercase-alignment ()
  "make all sequences lowercase"
  (interactive)
  (save-excursion
    (ralee-find-first-line)
    (while (< (point) (point-max))
      (if (ralee-is-alignment-line)
	  (progn 
	    (ralee-find-first-column)
	    (let ((first (point)))
	      (end-of-line)
	      (downcase-region first (point))
	      )
	    )
	)
      (forward-line)
      )
    )
  )


(defun t-to-u ()
  "convert all T's to U's"
  (interactive)
  (save-excursion
    (ralee-find-first-line)
    (while (< (point) (point-max))
      (if (ralee-is-alignment-line)
	  (progn 
	    (end-of-line)
	    (let ((end (point)))
	      (ralee-find-first-column)	      
	      ; I struggle to get perform-replace to work as advertised
	      ; so get round it with a narrow and widen
	      (save-restriction
		(narrow-to-region (point) end)
		(perform-replace "T" "U" nil nil nil)
		(goto-char (point-min))
		(perform-replace "t" "u" nil nil nil)
		)
	      )
	    )
	)
      (forward-line)
      )
    )
  )

(defun u-to-t ()
  "convert all U's to T's"
  (interactive)
  (save-excursion
    (ralee-find-first-line)
    (while (< (point) (point-max))
      (if (ralee-is-alignment-line)
	  (progn 
	    (end-of-line)
	    (let ((end (point)))
	      (ralee-find-first-column)	      
	      ; I struggle to get perform-replace to work as advertised
	      ; so get round it with a narrow and widen
	      (save-restriction
		(narrow-to-region (point) end)
		(perform-replace "U" "T" nil nil nil)
		(goto-char (point-min))
		(perform-replace "u" "t" nil nil nil)
		)
	      )
	    )
	)
      (forward-line)
      )
    )
  )



(provide 'ralee-edit)
