;;; ralee-paint

; Copyright (c) 2004 Sam Griffiths-Jones
;
; This is part of RALEE -- see
; http://www.sanger.ac.uk/Users/sgj/ralee/ and the 00README file
; that should accompany this file.
;
; RALEE is free software; you can redistribute it and/or modify it
; under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; RALEE is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
; General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with RALEE; if not, write to the Free Software Foundation,
; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA



(defun paint-line-by-pairs (pairs-list)
  "colour the current line according to the SS_cons line"
  (let ((pair ()) 
	open 
	close 
	openbase 
	closebase 
	helices
	line-start
	line-end)

    (beginning-of-line) (setq line-start (point))
    (end-of-line) (setq line-end (point))
    (put-text-property line-start line-end 'face 'default)

    (setq helices (ralee-helix-map pairs-list))

    (while pairs-list
      (setq pair (car pairs-list))
      (setq pairs-list (cdr pairs-list))

      (move-to-column (car pair))
      (setq open (point))
      (setq openbase (char-after))

      (move-to-column (cdr pair))
      (setq close (point))
      (setq closebase (char-after))

      ; remainder operator guarantees that we'll get a colour
      (setq face-num (% (cdr (assoc (car pair) helices)) (length ralee-faces)))

      (let ((case-fold-search 1)) ; case-independent search
	(if (or
	     (and (char-equal openbase ?C) (char-equal closebase ?G))
	     (and (char-equal openbase ?G) (or (char-equal closebase ?C)
					       (char-equal closebase ?U)
					       (char-equal closebase ?T)))
	     (and (char-equal openbase ?A) (or (char-equal closebase ?U)
					       (char-equal closebase ?T)))
	     (and (or (char-equal openbase ?U)
		      (char-equal openbase ?T)) (or (char-equal closebase ?A)
						    (char-equal closebase ?G))))

	    (progn
	      (put-text-property open (1+ open) 'face (nth face-num ralee-faces))
	      (put-text-property close (1+ close) 'face (nth face-num ralee-faces))
	      )
	  )
	)
      )
    )
  )



(defun paint-column-by-base ()
  "paint a column by base id"
  (interactive)
  (save-excursion
    (let (column
	  counts
	  (cons 0.3)
	  (mod-flag (buffer-modified-p))
	  )

      (setq column (current-column))
      (setq counts (ralee-count-bases-in-column))

      (if (or (> (/ (float (nth 1 counts)) (float (nth 0 counts))) cons)
	      (> (/ (float (nth 2 counts)) (float (nth 0 counts))) cons)
	      (> (/ (float (nth 3 counts)) (float (nth 0 counts))) cons)
	      (> (/ (float (nth 4 counts)) (float (nth 0 counts))) cons))

	  (progn
	    (goto-char (point-min))

	    (while (< (point) (point-max))  ; go round again colouring bases
	      (if (ralee-is-alignment-line)
		  (progn
		    (move-to-column column)
		    (setq base (char-after))
		    (let ((case-fold-search t))
		      (if (and (char-equal base ?A) (> (/ (float (nth 1 counts)) (nth 0 counts)) cons))
			  (put-text-property (point) (1+ (point)) 'face (nth 0 ralee-faces)))
		      (if (and (char-equal base ?C) (> (/ (float (nth 2 counts)) (nth 0 counts)) cons))
			  (put-text-property (point) (1+ (point)) 'face (nth 1 ralee-faces)))
		      (if (and (char-equal base ?G) (> (/ (float (nth 3 counts)) (nth 0 counts)) cons))
			  (put-text-property (point) (1+ (point)) 'face (nth 2 ralee-faces)))
		      (if (and (or (char-equal base ?T)
				   (char-equal base ?U)) (> (/ (float (nth 4 counts)) (nth 0 counts)) cons))
			  (put-text-property (point) (1+ (point)) 'face (nth 3 ralee-faces)))
		      )
		    )
		)
	      (forward-line)
	      )
	    )
	)
      (set-buffer-modified-p mod-flag)
      )
    )
  )


(defun paint-buffer-by-base ()
  "paint whole buffer by base id"
  (interactive)
  (save-excursion
    (let ((mod-flag (buffer-modified-p)))
      (put-text-property (point-min) (point-max) 'face 'default)
      (goto-char (point-min))
      (while (equal (ralee-is-alignment-line) nil)
	(forward-line)) ; search for the first alignment line
      (end-of-line)
      (let ((line-end (point)))
	(search-backward " ")
	(while (< (point) line-end)
	  (paint-column-by-base)
	  (forward-char)
	  )
	)
      (set-buffer-modified-p mod-flag)
      )
    )
  )



(defun paint-column-by-cons ()
  "paint a column by base conservation"
  (interactive)
  (save-excursion
    (let (column
	  counts
	  (mod-flag (buffer-modified-p))
	  )

      (setq column (current-column))
      (setq counts (ralee-count-bases-in-column))

      (if (or (> (/ (float (nth 1 counts)) (float (nth 0 counts))) 0.4)
	      (> (/ (float (nth 2 counts)) (float (nth 0 counts))) 0.4)
	      (> (/ (float (nth 3 counts)) (float (nth 0 counts))) 0.4)
	      (> (/ (float (nth 4 counts)) (float (nth 0 counts))) 0.4))

	  (progn
	    (goto-char (point-min))

	    (while (< (point) (point-max))  ; go round again colouring them
	      (if (ralee-is-alignment-line)
		  (progn
		    (move-to-column column)
		    (setq base (char-after))
		    (let ((case-fold-search t))
		      (if (or (and (char-equal base ?A) (> (/ (float (nth 1 counts)) (float (nth 0 counts))) 0.8))
			      (and (char-equal base ?C) (> (/ (float (nth 2 counts)) (float (nth 0 counts))) 0.8))
			      (and (char-equal base ?G) (> (/ (float (nth 3 counts)) (float (nth 0 counts))) 0.8))
			      (and (or (char-equal base ?U) 
				       (char-equal base ?T)) (> (/ (float (nth 4 counts)) (float (nth 0 counts))) 0.8)))

			  (put-text-property (point) (1+ (point)) 'face (nth 7 ralee-faces))

			(if (or (and (char-equal base ?A) (> (/ (float (nth 1 counts)) (float (nth 0 counts))) 0.6))
				(and (char-equal base ?C) (> (/ (float (nth 2 counts)) (float (nth 0 counts))) 0.6))
				(and (char-equal base ?G) (> (/ (float (nth 3 counts)) (float (nth 0 counts))) 0.6))
				(and (or (char-equal base ?U) 
					 (char-equal base ?T)) (> (/ (float (nth 4 counts)) (float (nth 0 counts))) 0.6)))

			    (put-text-property (point) (1+ (point)) 'face (nth 0 ralee-faces))

			  (if (or (and (char-equal base ?A) (> (/ (float (nth 1 counts)) (float (nth 0 counts))) 0.4))
				  (and (char-equal base ?C) (> (/ (float (nth 2 counts)) (float (nth 0 counts))) 0.4))
				  (and (char-equal base ?G) (> (/ (float (nth 3 counts)) (float (nth 0 counts))) 0.4))
				  (and (or (char-equal base ?U) 
					   (char-equal base ?T)) (> (/ (float (nth 4 counts)) (float (nth 0 counts))) 0.4)))

			      (put-text-property (point) (1+ (point)) 'face (nth 8 ralee-faces))

			    )
			  )
			)
		      )
		    )
		)
	      (forward-line)
	      )
	    )
	)
      (set-buffer-modified-p mod-flag)
      )
    )
  )


(defun paint-buffer-by-cons ()
  "paint whole buffer by conservation"
  (interactive)
  (save-excursion
    (let ((mod-flag (buffer-modified-p)))
      (put-text-property (point-min) (point-max) 'face 'default)
      (goto-char (point-min))
      (ralee-find-first-line)
      (end-of-line)
      (let ((line-end (point)))
	(search-backward " ")
	(while (< (point) line-end)
	  (paint-column-by-cons)
	  (forward-char)
	  )
	)
      (set-buffer-modified-p mod-flag)
      )
    )
  )



(defun paint-line-by-ss ()
  "get the structure, and then paint the current line"
  (interactive)
  (save-excursion
    (let (line
	  pairs
	  (mod-flag (buffer-modified-p)))
      (setq line (ralee-get-ss-line))
      (setq pairs (ralee-get-base-pairs line))
      (paint-line-by-pairs pairs)
      (set-buffer-modified-p mod-flag)
      )
    )
  )


(defun paint-buffer-by-ss ()
  "get the structure, and then paint the whole buffer"
  (interactive)
  (save-excursion
    (let (line
	  pairs
	  (mod-flag (buffer-modified-p))
	  )
      (put-text-property (point-min) (point-max) 'face 'default)
      (setq line (ralee-get-ss-line))
      (setq pairs (ralee-get-base-pairs line))
      (beginning-of-buffer)
      (while (< (point) (point-max))
	(if (ralee-is-alignment-line)
	    (paint-line-by-pairs pairs))
	(forward-line)
	)
      (set-buffer-modified-p mod-flag)
      )
    )
  )

(defun paint-buffer-by-current-ss-line ()
  "get structure from the current line, and then paint the whole buffer"
  (interactive)
  (save-excursion
    (let (line
	  pairs
	  (mod-flag (buffer-modified-p))
	  )
      (put-text-property (point-min) (point-max) 'face 'default)
      (setq line (ralee-get-current-ss-line))
      (if line
	  (progn
	    (setq pairs (ralee-get-base-pairs line))
	    (beginning-of-buffer)
	    (while (< (point) (point-max))
	      (if (ralee-is-alignment-line)
		  (paint-line-by-pairs pairs))
	      (forward-line)
	      )
	    )
	(message "Current line doesn't contain structure markup")
	)
      (set-buffer-modified-p mod-flag)
      )
    )
  )


(provide 'ralee-paint)
